#include <PA9.h> 
#include "tex1_pcx.h"
#include "tex2_pcx.h"

// This adds the files. No other loading needed.
#include "3ds/3dsLoader.h"

float cur_time;


typedef struct
{
	float x,y,z,angle,gravity;
}tsVertex;

// player position
tsVertex player;
// camera position
tsVertex cam,cam_dest;
// mushrooms
tsVertex mushroom[5];


void UpdateCamera()
{
	// camera follows player
	cam_dest.x=player.x;
	cam_dest.y=player.y/2+1;
	cam_dest.z=player.z+3;
	
	// this makes the camera move softly to its destination
	cam.x-=(cam.x-cam_dest.x)/20;
	cam.y-=(cam.y-cam_dest.y)/10;
	cam.z-=(cam.z-cam_dest.z)/20;
}
bool collision(float x,float y,float z)
{
	// simple bounding box collision

	int i=0;
	for(i=0;i<=5;i++)
	{
		if(x>mushroom[i].x-0.8 && x<mushroom[i].x+0.8)
		{
			if(z>mushroom[i].z-0.8 && z<mushroom[i].z+0.8)
			{
				if(y>mushroom[i].y-0.001 && y<mushroom[i].y+1)
				{
					return 1;
				}
			}
		}
	
	}

	return 0;
}
void UpdatePlayer()
{
	float spd=0.15;
	
	// moving
	if(Pad.Held.Left)
	{
		if(!collision(player.x-spd,player.y,player.z))
			player.x-=spd;
		player.angle=90;
	}
	if(Pad.Held.Right)
	{
		if(!collision(player.x+spd,player.y,player.z))
			player.x+=spd;
		player.angle=-90;
	}
	if(Pad.Held.Up)
	{
		if(!collision(player.x,player.y,player.z-spd))
			player.z-=spd;
		player.angle=180;
		
		if(Pad.Held.Left)player.angle-=45;
		if(Pad.Held.Right)player.angle+=45;
	}
	if(Pad.Held.Down)
	{
		if(!collision(player.x,player.y,player.z+spd))
			player.z+=spd;
		player.angle=0;
		
		if(Pad.Held.Left)player.angle+=45;
		if(Pad.Held.Right)player.angle-=45;
	}	
	
	// Jumping and gravity
	if(player.y<=0)
	{
		player.y=0;
			
		player.gravity=0;
		
		if(Pad.Held.A)
			player.gravity=0.5;
	}
	else
		player.gravity-=0.05;
		
	// bouncy mushrooms
	if(collision(player.x,player.y+player.gravity,player.z))
	{
		if(Pad.Held.A)
			player.gravity*=-1.05;
		else
			player.gravity*=-0.5;
	}	
		player.y+=player.gravity;
}

int main()
{
	PA_Init();
	PA_InitVBL();
	PA_Init3D();
	
	u8 timmy=NewTimer(1);
	StartTime(1);
	StartTimer(timmy);

	PA_LoadTexture(1,(u8*)tex1_pcx,texPCX);// main character
	PA_LoadTexture(2,(u8*)tex2_pcx,texPCX);// mushroom
	
	// Set mushroom positions
	mushroom[0].x=-2;
	mushroom[0].y=0;
	mushroom[0].z=-2;
	
	mushroom[1].x=-2;
	mushroom[1].y=1;
	mushroom[1].z=-2;
	
	mushroom[2].x=-1.5;
	mushroom[2].y=0;
	mushroom[2].z=-2;
	
	mushroom[3].x=2;
	mushroom[3].y=0;
	mushroom[3].z=0;
	
	mushroom[4].x=1;
	mushroom[4].y=0;
	mushroom[4].z=1;
	
	mushroom[5].x=0;
	mushroom[5].y=0;
	mushroom[5].z=-1;
	
	PA_SwitchScreens();

	
	while(1) 
	{

		cur_time=(float)Tick(timmy)/2;
			
		PA_Init3DDrawing(0.0,0.0,1.0,//x,y,z
		0.0,0.0,0.0);//eyex, eyey, eyez
		
		// Setup lighting
			glLight(0,RGB15(31,31,31),0,0,floatov10(-1.0));
			glPushMatrix();
			glMatrixMode(GL_TEXTURE);
			glIdentity();
			glMatrixMode(GL_MODELVIEW);
			glMaterialf(GL_AMBIENT, RGB15(3,3,3));
			glMaterialf(GL_EMISSION, RGB15(7,7,7));
			glMaterialf(GL_DIFFUSE, RGB15(12,12,12));
			glMaterialf(GL_SPECULAR, BIT(15) | RGB15(31,31,31));
			glMaterialShinyness();
			glMatrixMode(GL_MODELVIEW);
			glPolyFmt(POLY_ALPHA(31) | POLY_CULL_NONE  | POLY_FORMAT_LIGHT0); // this enables lightin
			glPushMatrix();	
		
		
		UpdatePlayer();
		UpdateCamera();
		
		
		
		// Draw the player
		PA_Texture(1);
		glColor3f(1,1,1);
		
		glLoadIdentity();
		PA_glulookat(cam.x,cam.y,cam.z,player.x,player.y,player.z,0,1,0);	
		glTranslatef(player.x,player.y,player.z);
		glRotatef(player.angle,0,1,0);
		
		Draw3DS(green_3DS);
		
		// draw the mushrooms
		PA_Texture(2);
		
		int i=0;
		for(i=0;i<=5;i++)
		{
			glLoadIdentity();
			PA_glulookat(cam.x,cam.y,cam.z,player.x,player.y,player.z,0,1,0);	
			glTranslatef(mushroom[i].x,mushroom[i].y,mushroom[i].z);			
			Draw3DS(mushroom_3DS);
		}

		// draw ground
		glPolyFmt(POLY_ALPHA(31) | POLY_CULL_NONE); //disable lights
		
		PA_Texture(0);
		glLoadIdentity();
		PA_glulookat(cam.x,cam.y,cam.z,player.x,player.y,player.z,0,1,0);	
		
		glColor3f(0,1,0);
		glBegin(GL_QUADS);
			glVertex3f(-5,0, 5);
			glVertex3f( 5,0, 5);
			glVertex3f( 5,0,-5);
			glVertex3f(-5,0,-5);		
		glEnd();
		
		
		
		/////////////////////////////////////
			glFlush();PA_WaitForVBL();

	}
	return 1;
}

